
// JSonSerial2: C++ Object Serialization in JSON
// See: https://www.telecom-paris.fr/~elc/software/
// This file should be included *after* jsonserial.hpp when serializing std::set
//
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com)
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef jsonserial_set_hpp
#define jsonserial_set_hpp
#include <set>
namespace jsonserial {

template <class T, class Comp, class Alloc>
struct is_std_set<std::set<T,Comp,Alloc>> : std::true_type {};

template <class T, class Comp, class Alloc>
struct is_std_set<std::multiset<T,Comp,Alloc>> : std::true_type {};

template<class T>
struct JSonArrayImpl<T, typename std::enable_if<is_std_set<T>::value>::type>
: public JSonArray {

  JSonArrayImpl(T& set) : set_(set) {set_.clear();}

  void add(JSonSerial& js, JSonClass::Creator* cr, JKString& s) override {
    typename T::value_type val;
    JObjectPtr* objptr{};
    readArrayValue(js, val, objptr, cr, s);
    set_.insert(val);
  }

  T& set_;
};

}
#endif
